package cn.iocoder.yudao.module.bpm.service.oa;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.loan.BpmOALoanCreateReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOALoanDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOALoanMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.service.task.BpmHistoryProcessInstanceService;
import cn.iocoder.yudao.module.bpm.service.task.BpmProcessInstanceService;
import cn.iocoder.yudao.module.system.api.loan.LoanApi;
import cn.iocoder.yudao.module.system.api.loan.dto.LoanDTO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.flowable.engine.runtime.ProcessInstance;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_LOAN_NOT_EXISTS;

/**
 * OA 加班申请 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class BpmOALoanServiceImpl extends BpmOABaseService implements BpmOALoanService {

    /**
     * OA 加班对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_loan_2";

    @Resource
    private BpmOALoanMapper loanMapper;

    @Resource
    private BpmProcessInstanceService processInstanceService;

    @Resource
    private BpmHistoryProcessInstanceService historyProcessInstanceService;

    @Resource
    private LoanApi loanApi;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createLoan(Long userId, BpmOALoanCreateReqVO vo) {
        //插入OA 借支申请
        BpmOALoanDO loan = BeanUtils.toBean(vo, BpmOALoanDO.class)
                .setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        loanMapper.insert(loan);

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceService.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(loan.getId())));

        // 将工作流的编号，更新到 OA 借支申请单中
        loanMapper.updateById(new BpmOALoanDO().setId(loan.getId()).setProcessInstanceId(processInstanceId));

        // 判断是否为重新发起的流程
        if (vo.getProcessInstanceId() != null && vo.getResult() == 3) {
            historyProcessInstanceService.createHistoryProcessInstance(processInstanceId, vo.getProcessInstanceId());
        }
        List<UploadUserFile> fileItems = vo.getFileItems();
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId, fileItems);
        }
        return loan.getId();
    }

    @Override
    public void updateLoanResult(String processInstanceId, Long id, Integer result) {
        BpmOALoanDO loanDO = validateLeaveExists(id);
        loanMapper.updateById(new BpmOALoanDO().setId(id).setResult(result));

        //审核通过 （最后节点）
        if (BpmProcessInstanceResultEnum.APPROVE.getResult().equals(result)) {

            ProcessInstance instance = processInstanceService.getProcessInstance(processInstanceId);

            if (instance.isEnded()) {

                // 同步插入借支表中
                LoanDTO createDO = new LoanDTO()
                        .setAmount(loanDO.getTotalMoney())
                        .setReturnAmount(BigDecimal.ZERO);
                loanApi.createLoan(createDO);
            }
        }
    }

    private BpmOALoanDO validateLeaveExists(Long id) {
        BpmOALoanDO loanDO = loanMapper.selectById(id);
        if (loanDO == null) {
            throw exception(OA_LOAN_NOT_EXISTS);
        }
        return loanDO;
    }

    @Override
    public BpmOALoanDO getLoan(Long id) {

        return loanMapper.selectById(id);
    }

    @Override
    public BpmOALoanDO getByProcessInstanceId(String processInstanceId) {

        List<BpmOALoanDO> bpmOALoanDOS = loanMapper.selectList(BpmOALoanDO::getProcessInstanceId, processInstanceId);
        if (!CollectionUtil.isEmpty(bpmOALoanDOS)) {
            return bpmOALoanDOS.get(0);
        }
        return null;
    }

    @Override
    public List<BpmOALoanDO> getListByStaffId(Long staffId) {
        return loanMapper.selectList(new LambdaQueryWrapperX<BpmOALoanDO>()
                .eq(BpmOALoanDO::getSfUserId, staffId)
                .eq(BpmOALoanDO::getResult, BpmProcessInstanceResultEnum.APPROVE.getResult()));
    }
}
